#%% libraries
import numpy as np
import scipy.optimize as sci

#%% Impementation of two phase split using Rachford-Rice


def newton_rhapson(f, df, x0, tol):  # works,  but may fail to converge
    if abs(f(x0)) < tol:
        return x0
    else:
        return newton_rhapson(f, df, x0 - f(x0) / df(x0), tol)


def samesign(a, b):
    return a * b > 0


def bisect(f, low, high, tol=None):
    assert not samesign(f(low), f(high))

    for i in range(100):
        mid = (low + high) / 2
        if samesign(f(low), f(mid)):
            low = mid
        else:
            high = mid
        if tol is not None and abs(high - low) < tol:
            break
        return mid


def Fv_solver(f, df, x0, tol, Ki):
    fvmin = 1 / (1 - max(Ki))
    fvmax = 1 / (1 - min(Ki))

    if abs(f(x0)) < tol:
        if x0 < fvmax and x0 > fvmin:
            return x0
        else:
            # return sci.bisect(f, fvmin, fvmax)
            return bisect(f, fvmin, fvmax, 1e-4)
    else:
        if x0 < fvmax and x0 > fvmin:
            return newton_rhapson(f, df, x0 - f(x0) / df(x0), tol)
        else:
            # return sci.bisect(f, fvmin, fvmax)
            return bisect(f, fvmin, fvmax, 1e-4)


# def Ki_modified(p, T, pk, Tc, pc, AF):
#     # eq 4.42
#     # p = pressure, bara
#     # T = temperature, C
#     # pk = convergene pressure, bara
#     # Tc = critical temperature, K
#     # Pc = critical pressure, bara
#     # AF = acentric factor

#     Tr = (T + 273.15) / Tc
#     pr = p / pc
#     A2 = 0.8
#     A1 = 1 - (p / pk) ** A2
#     return (pc / pk) ** (A1 - 1) * np.exp(5.37 * A1 * (1 + AF) * (1 - 1 / Tr)) / pr


def Ki(p, T, Tc, pc, AF):
    # eq 4.42
    # p = pressure, bara
    # T = temperature, C
    # pk = convergene pressure, bara
    # Tc = critical temperature, K
    # Pc = critical pressure, bara
    # AF = acentric factor

    Tr = (T + 273.15) / Tc
    pr = p / pc
    return np.exp(5.37 * (1 + AF) * (1 - 1 / Tr)) / pr


def xi(zi, Fv, Ki):
    # eq 4.41
    return zi / (Fv * (Ki - 1) + 1)


def RR_equation(zi, Ki, Fv):
    # eq 4.36, not summed to zero
    return (zi * (Ki - 1)) / (1 + Fv * (Ki - 1))


#%% Peng Robinson implementation
def a(R, Tc, pc, alpha):
    # eq 4.21
    return 0.457236 * (R ** 2) * (Tc ** 2) * alpha / pc


def b(R, Tc, pc):
    # eq 4.21
    return 0.0777961 * R * Tc / pc


def m(AF):
    # eq 4.21 - 4.22
    if AF < 0.4:  # > 0.4 in monograph...
        return 0.37464 + 1.54226 * AF - 0.26992 * (AF ** 2)
    else:
        return 0.3796 + 1.485 * AF - 0.1644 * (AF ** 2) + 0.01667 * (AF ** 3)


def alpha(m, Tr):
    return (1 + m * (1 - Tr ** 0.5)) ** 2


def A_mix(A_vector, BIP, comp):
    N_a = len(A_vector)
    N_bip_0 = np.shape(BIP[0])[0]
    N_bip_1 = np.shape(BIP[1])[0]
    N_comp = len(comp)

    if N_bip_0 != N_bip_1 or N_a != N_bip_0 or N_a != N_comp:
        amix = "ERROR"
    else:
        amix = 0
        for i in range(N_comp):  # len(1, N_comp)
            for j in range(N_comp):
                amix = amix + comp[i] * comp[j] * (1 - BIP[i, j]) * (
                    (A_vector[i] * A_vector[j]) ** 0.5
                )
    return amix


def B_mix(B_vector, comp):
    return np.sum((comp * B_vector))


def Awi(A_vector, BIP, comp, k):
    # k is component index
    N_a = len(A_vector)
    N_bip_0 = np.shape(BIP[0])[0]
    N_bip_1 = np.shape(BIP[1])[0]
    N_comp = len(comp)

    if N_bip_0 != N_bip_1 or N_a != N_bip_0 or N_a != N_comp:
        awi = "ERROR"
    else:
        awi = 0
        i = k
        for j in range(N_comp):
            awi = awi + comp[j] * (1 - BIP[i, j]) * ((A_vector[i] * A_vector[j]) ** 0.5)
        return awi


def fugacity(p, R, T, z, A_mix, B_mix, Bi, yi, Awi, ci):
    fugacity_i = (
        yi
        * p
        * np.exp(
            Bi / B_mix * (z - 1)
            - np.log(z - B_mix)
            + A_mix
            / (2 * (2 ** 0.5) * B_mix)
            * (Bi / B_mix - 2 * Awi / A_mix)
            * np.log((z + ((2 ** 0.5) + 1) * B_mix) / (z + (-(2 ** 0.5) + 1) * B_mix))
        )
    ) * np.exp(-ci * p / (R * T))
    return fugacity_i


def Gibbs_energy(fugacity_vector, comp):
    return np.sum((comp * np.log(fugacity_vector)))


def SSQ_fugacity(Fv, Fl):
    return np.sum((Fl / Fv - 1) ** 2)


# %%
