import numpy as np


def ZfacStanding(p1, T1, Yg, unit):

    # ZStanding : Calculation of Z-factor based on Hall&Yarborough equation fitted to Standing-Katz Chart.
    #     P : Pressure (psia/bara)
    #     T : Temperature (oF/oC)
    #     Yg : Gas Specific Gravity (air=1)
    #     Unit:
    #           1 : Field
    #           2 : Metric

    #      Calculating pseudocritical temperature and pressure
    Tpc = 169.2 + 349.5 * Yg - 74 * Yg ** 2
    Ppc = 756.8 - 131 * Yg - 3.6 * Yg ** 2
    # Unit conversion
    if unit == 2:
        T = 9 / 5 * T1 + 32
    else:
        T = T1

    if unit == 2:
        P = 14.5038 * p1
    else:
        P = p1

    Tpr = (T + 460) / Tpc
    Ppr = P / Ppc
    T = 1 / Tpr
    a = 0.06125 * T * np.exp(-1.2 * (1 - T) ** 2)
    y = 0.001
    i = 0
    ff = 1

    while abs(ff) > 0.00000001 and i < 100:
        fy = (
            -a * Ppr
            + (y + y ** 2 + y ** 3 - y ** 4) / (1 - y) ** 3
            - (14.76 * T - 9.76 * T ** 2 + 4.58 * T ** 3) * y ** 2
            + (90.7 * T - 242.2 * T ** 2 + 42.4 * T ** 3) * y ** (2.18 + 2.82 * T)
        )
        dfY = (
            (1 + 4 * y + 4 * y ** 2 - 4 * y ** 3 + y ** 4) / (1 - y) ** 4
            - (29.52 * T - 19.52 * T ** 2 + 9.16 * T ** 3) * y
            + (2.18 + 2.82 * T)
            * (90.7 * T - 242.2 * T ** 2 + 42.4 * T ** 3)
            * y ** (1.18 + 2.82 * T)
        )
        y = y - fy / dfY
        ff = fy / dfY
        i = i + 1

        Z = a * Ppr / y
    return Z


def Bg(p, T, Z, unit):
    # Unit:
    # 1 : Field     'Bg is in cf/scf
    # 2 : Metric     'Bg is in cm/scm
    # p, pressure, psia or bara
    # T, temperature, F or C
    # Z gas deviation factor
    # returns gas volume factor, m3/Sm3
    if unit == 1:
        return Z * (14.7 / p) * (T + 460) / (60 + 460)
    else:
        return Z * (1.01325 / p) * (T + 273.15) / (15.56 + 273.15)


def deng(p, T, Z, sgg, unit):
    # p in bara
    # T in C
    # returns gas density, kg/m3
    if unit == 2:
        T1 = T + 273.15
        return p / (Z * (0.083143 / (sgg * 28.97)) * T1)
    else:
        T1 = (T - 32 / 1.8) + 273.15
        p = 14.5 * p
        return p / (Z * (0.083143 / (sgg * 28.97)) * T1)


def visg(sgg, deng, T):
    # The gas viscosity (Visg cp) is calculated from the correlation of Lee et al.
    # deng in kg/m3
    # T in C
    Tl = 9 / 5 * T + 32
    DLG = deng / 1000
    m = 28.97 * sgg
    TR = Tl + 460

    k = (9.4 + 0.02 * m) * TR ** 1.5 / (209 + 19 * m + TR)
    X = 3.5 + 986 / TR + 0.01 * m
    Y = 2.4 - 0.2 * X

    return 0.0001 * k * np.exp(X * DLG ** Y)


