#%%
import SALib
import numpy as np
import matplotlib.pyplot as plt
import datapreprocessing as dpp
import gas
import water

# %%

dataset = dpp.getPPLData("../OLGA/", "1.ppl")
# %%
time = dataset[0]
ID = dataset[1]
# %%
branch = ID[0]
flow_regime_ID = ID[1][-1]

unique = np.unique(flow_regime_ID, return_index=True, return_counts=True)
loaded_flow = 0
if 3 in unique[0]:
    idx3 = np.abs(np.array(unique[0]) - 3).argmin()
    start3 = unique[1][idx3]
    count3 = unique[2][idx3]
    loaded_flow += count3
if 4 in unique[0]:
    idx4 = np.abs(np.array(unique[0]) - 4).argmin()
    start4 = unique[1][idx4]
    count4 = unique[2][idx4]
    loaded_flow += count4

loaded_length = (branch[2] - branch[1]) * loaded_flow

# %% Liquid height in each timestep from average densities and integrated gravitational pressure
L_tubing = 3000
T = 70
SG_gas = 0.609
SG_water = 1.02
deng_avg = []
denw_avg = []
#%%
for i in dataset[8][1]:  # Pressure along branch
    PTb = i
    Zg = [gas.ZfacStanding(i, T, SG_gas, 2) for i in PTb * 1e-5]
    deng = [gas.deng(PTb[i] * 1e-5, T, Zg[i], SG_gas, 2) for i in range(len(PTb))]
    deng_avg.append(np.average(deng))

    Bw = [water.water_properties(i, T, 15.56) for i in PTb * 1e-5]
    denw = [water.denw1(SG_water, SG_gas, i) * 1000 for i in Bw]
    denw_avg.append(np.average(denw))

for i in dataset[6][1]:  # integrated gravitational pressure along branch
    dpzg = i[-1]  # total integrated gravity pressure drop in timestep i
    LIQHeight = (dpzg / 9.81 - np.array(deng_avg) * L_tubing) / (
        np.array(denw_avg) - np.array(deng_avg)
    )


# %% see liquid loading from superf
UG = dataset[2][1][-1][1]  # velocity in the bottom
UL = dataset[3][1][-1][1]  # velocity in the bottom
DPZA = dataset[4][1][-1][-1]
DPZF = dataset[5][1][-1][-1]
HOL = dataset[7][1][-1]

#%% concept - integrate velocity along branch and time
UG = dataset[3][1]
branch = dataset[3][0]
sum1 = []
for i in UG:
    sum1.append(np.trapz(i, branch))

#%% predicative algorithm - polynomial regression
# x = time/86400
# y = QWST
# poly = PolynomialFeatures(degree, include_bias=False)
# poly_features = poly.fit_transform(x.reshape(-1,1))
# poly_reg_model = LinearRegression()
# poly_reg_model.fit(poly_features, y)
# y_predicted = poly_reg_model.predict(poly_features)
# plt.scatter(x,y)
# plt.plot(x,y_predicted, c="red")

#%% sensitivity analysis
from SALib.analyze import sobol
from SALib.sample import saltelli


def vc_turner(liquid_density, gas_density, surface_tension, pwh=100):
    # surf tens in dyne/cm, dens in kg/m3, critical velocity returned in m/s
    if pwh >= 800:
        k = 1.92  # turner value
    else:
        k = 1.59  # Coleman value

    rhol = liquid_density * 0.0624279605761452
    rhog = gas_density * 0.0624279605761452

    # return 20.4*( surface_tension*(rhol - rhog)/rho_g**2)**(0.25)*0.3048 #1.8949*
    return k * (surface_tension * (rhol - rhog) / rhog ** 2) ** (0.25) * 0.3048


def Qc(p1, p2, T1, T2, Yg, Yw, d, unit):
    if unit == 1:
        p1 = p1 / 14.5
        p2 = p2 / 14.5
        T1 = (T1 - 32) / 1.8
        T2 = (T2 - 32) / 1.8

    A = np.pi / 4 * (0.0254 * d) ** 2  # m2

    Z1 = gas.ZfacStanding(p1, T1, Yg, 2)
    Z2 = gas.ZfacStanding(p2, T2, Yg, 2)

    Bw = water.water_properties(p1, T1, T2)  # m3/Sm3
    Rsw = water.Rsw(p1)[0]

    rho_g = gas.deng(p1, T1, Z1, Yg, 2)  # kg/m3
    rho_l = water.denw1(Yw, Yg, Bw, Rsw) * 1000  # kg/m3
    sigma = water.SurfTens_wg(T1, rho_l / 1000, rho_g / 1000)
    vc = vc_turner(rho_l, rho_g, sigma, p1)  # m/s
    Qc = (p1 * vc * A * Z2 * (T2 + 273.15)) / (Z1 * p2 * (T1 + 273.15)) * 86400  # Sm3/d
    if unit == 1:
        Qc = Qc / 1000 * 35.3146667
        vc = vc / 0.3048
    return Qc, vc


problem = {
    "num_vars": 7,
    "names": ["p1", "p2", "T1", "T2", "Yg", "Yw", "d"],
    "bounds": [
        [1, 100],
        [0.97, 1.3],
        [60, 100],
        [10, 20],
        [0.5, 1],
        [0.96, 1.04],
        [1.25, 3.5],
    ],
}

param_values = saltelli.sample(problem, 1024)

Y = np.zeros([param_values.shape[0]])
for i, X in enumerate(param_values):
    p1, p2, T1, T2, Yg, Yw, d = X
    Y[i] = Qc(p1, p2, T1, T2, Yg, Yw, d, 2)[0]

Si = sobol.analyze(problem, Y)

print(Si["S1"], Si["ST"])
total_Si, first_Si, second_Si = Si.to_df()
Si.plot()

# %%
# from mpl_toolkits import mplot3d
# from mpl_toolkits.mplot3d import axes3d
# import numpy as np
# import matplotlib.pyplot as plt

# time, ID, UG, UL, DPZA, DPZF, DPZG, HOL, PTb = getPPLData("../OLGA/", "13.ppl")

# x = ID[0]
# # x = np.linspace(0,3000, 481)
# # y = [ID[1][i] for i in range(len(time))]
# y = ID[1]
# # z = time
# z = np.linspace(time[0], time[-1], 51)

# fig = plt.figure()
# ax = plt.axes(projection="3d")
# ax.set_xlabel("Lenght along branch [m]")
# ax.set_zlabel("ID")
# ax.set_ylabel("Time [s]")
# for i in range(len(y)):
#     ax.plot3D(x,z,y[i])

# # ax.plot_surface(x,z,y)
# # ax.plot3D(ID[0], ID[1][0])
