#%%
import numpy as np

# import matplotlib.pyplot as plt
import scipy.optimize as so
from sklearn.preprocessing import PolynomialFeatures
from sklearn.linear_model import LinearRegression
from sklearn.metrics import r2_score

"""Calculate injectivity index  and injection performance relationship of a formation, vertical well. Use equation for undersaturated 
    oil vertical oil well with radial geometry"""

# # Data
# re = 500  # m
# rw = 0.15  # m
# kh = 10  # md, horizontal permeability
# t = 20  # m, layer thickness
# Pr = 300  # bara
# Tr = 80  # degC
# s = 0  # skin
# csw = 17.1 / (1e6 * 150000 ** (-1) - 1)  # ppm

# Pwf = np.linspace(300, 500, 50)  # bar


# %% Equations


def water_properties(pr, Tr, T_0, ws=0.05):

    csw = 17.1 / (1e6 * (ws * 1e6) ** (-1) - 1)  # ppm

    def fvfsolver1(guess):

        [Bw0] = guess
        T = Tr + 273.15
        T0 = T_0 + 273.15  # 15.56 + 273.15
        A0 = (
            5.91635
            - 0.01035794 * T
            + (0.9270048e-5) * T ** 2
            - 1127.552 * T ** (-1)
            + 100674.1 * T ** (-2)
        )
        A1 = -2.5166 + 0.0111766 * T - (0.170552e-4) * T ** 2
        A2 = 2.84851 - 0.0154305 * T + (0.223982e-4) * T ** 2
        A00 = (
            5.91635
            - 0.01035794 * T0
            + (0.9270048e-5) * T0 ** 2
            - 1127.552 * T0 ** (-1)
            + 100674.1 * T0 ** (-2)
        )
        A10 = -2.5166 + 0.0111766 * T0 - (0.170552e-4) * T0 ** 2
        A20 = 2.84851 - 0.0154305 * T0 + (0.223982e-4) * T0 ** 2
        vw = A00 + A10 * ws + A20 * ws ** 2
        vw0 = A0 + A1 * ws + A2 * ws ** 2
        eq1 = Bw0 - vw0 / vw
        return [eq1]

    def fvfsolver2(guess):
        Bw0 = so.root(fvfsolver1, [0.9])["x"][0]
        [Bw] = guess
        p = pr * 14.5
        T = Tr * 9 / 5 + 32
        A0 = 1e6 * (0.314 + 0.58 * ws + 1.9e-4 * T - 1.45e-6 * T ** 2)
        A1 = 8 + 50 * ws - 0.125 * ws * T
        eq1 = Bw - Bw0 * (1 + A1 / A0 * p) ** (-1 / A1)
        return [eq1]

    # vis_water = 0.354 #cp, @80 degC

    def viscsolver(guess):
        a11 = 3.324e-2
        a12 = 3.624e-3
        a13 = -1.879e-4
        a21 = -3.96e-2
        a22 = 1.02e-2
        a23 = -7.02e-4
        a31 = 1.2378
        a32 = -1.303e-3
        a33 = 3.060e-6
        a34 = 2.55e-8
        visw20 = 1.002
        T = Tr  # degC

        visw = guess[0]
        visww = guess[1]
        A0 = 1e-3 * (0.8 + 0.01 * (T - 90) * np.exp(-0.25 * csw))
        visw0 = guess[2]

        A1 = a11 * csw + a12 * csw ** 2 + a13 * csw ** 3
        A2 = a21 * csw + a22 * csw ** 2 + a23 * csw ** 3

        # eq3 = A0 - 1e-3 * (0.8 + 0.01 * (T - 90) * np.exp(-0.25 * csw))
        eq4 = np.log10(visw0 / visw20) - (
            a31 * (20 - T) ** 1 / (96 + T)
            + a32 * (20 - T) ** 2 / (96 + T)
            + a33 * (20 - T) ** 3 / (96 + T)
            + a34 * (20 - T) ** 4 / (96 + T)
        )
        eq5 = visw - (1 + A0 * p) * visww
        eq6 = np.log10(visww / visw0) - A1 - A2 * np.log10(visw0 / visw20)

        return [eq4, eq5, eq6]

    # vis = so.root(viscsolver, [0.4, 0.8, 0.5])["x"][0]
    fvf = so.root(fvfsolver2, [0.9])["x"][0]

    return fvf  # ,vis


# Bw = water_properties(10, 70, 15000)
# %%


def Bw1(p, T, pref=1.01325, Tref=15.56, Bwref=1, C=0):
    # p in bara
    # T in C
    # C in g/L

    p = p * 14.5
    T = 1.8 * T + 32

    cw = (7.033 * p + 541.5 * C - 537.9 * T + 403.3e3) ** (-1)
    cw_ref = (7.033 * p + 541.5 * C - 537.9 * T + 403.3e3) ** (-1)

    return Bwref * (1 - cw_ref * (p - pref))


def denw1(SGw, SGg, Bw, Rs=0, unit=2):
    if unit == 1:
        return (62.336 * SGw + 0.0136 * SGg * Rs) / Bw
    else:
        return (0.999012 * SGw + 0.000218 * SGg * Rs) / Bw  # SG


def SurfTens_wg(T, rho_wg, rho_hg, Cs=0):
    # T in C, rho in g/cc
    # surf tens is in dynes/cm

    T = 1.8 * T + 32 + 460
    Tc = 302.881
    A = 1.53988 * (rho_wg - rho_hg) + 2.08339
    B = T / Tc
    C = 0.821976 - 1.83785e-3 * T + 1.34016e-6 * T ** 2
    D = 3.6667
    if Cs == 0:
        return (A / B ** (C)) ** D
    else:
        return (A / B ** (C)) ** D + 3.44e-5 * Cs


def water_solubility(p, T):
    p = 14.5 * p
    T = 1.8 * T + 32

    A0 = 0.299
    A01 = -1.273e-3
    A02 = 0.000
    A03 = 0.000
    A10 = 2.283e-3
    A11 = -1.870e-5
    A12 = 7.949e-8
    A13 = -7.881e-11
    A20 = -2.850e-7
    A21 = 2.72e-9
    A22 = -1.123e-11
    A23 = 1.361e-14
    A30 = 1.181e-11
    A31 = -1.082e-13
    A32 = 4.275e-16
    A33 = -4.846e-19

    A0_03 = [0.299, -1.273e-3, 0.000, 0.000]
    A1_03 = [2.283e-3, -1.870e-5, 7.949e-8, -7.881e-11]
    A2_03 = [-2.850e-7, 2.72e-9, -1.123e-11, 1.361e-14]
    A3_03 = [1.181e-11, -1.082e-13, 4.275e-16, -4.846e-19]
    A = [A0_03, A1_03, A2_03, A3_03]

    sumlist = []
    a = 0
    for i in range(len(A)):
        for j in range(len(A)):
            a = a + T ** j * A[i][j]
            if j == len(A) - 1:
                sumlist.append(a * p ** i)
                a = 0

    xC1 = 1e-3 * np.sum(A)

    Rsw_sc = 7370 * xC1 / (1 - xC1) * 6.2898

    return Rsw_sc


# %%


def Rsw(p):
    px = (
        np.array(
            [
                0,
                484.642,
                976.109,
                1467.577,
                1972.696,
                2928.328,
            ]
        )
        / 14.5
    )
    GORx = (
        np.array([0, 3.780, 7.260, 10.469, 13.351, 17.811]) * 6.2898
    )  # webplotdigitizer of Fig 9.6 in Phase Behavior

    x = px
    y = GORx
    degree = 3

    poly = PolynomialFeatures(degree, include_bias=False)
    poly_features = poly.fit_transform(x.reshape(-1, 1))
    poly_reg_model = LinearRegression()
    poly_reg_model.fit(poly_features, y)
    # y_predicted = poly_reg_model.predict(poly_features.reshape(-1,1))

    p = np.array(p)
    Rs_features = poly.fit_transform(p.reshape(-1, 1))
    Rs_pred = poly_reg_model.predict(Rs_features)

    return Rs_pred


# %%
